#include "linuxinputevent.h"
#include <QString>
#include <QApplication>
#include <linux/input.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>
#include <QDebug>

LinuxInputEvent::LinuxInputEvent(const QString &dev, QObject *parent)
	: QObject(parent)
{
	m_fd = ::open((const char *)dev.toLocal8Bit(), O_RDWR | O_NONBLOCK);
	if (m_fd < 0) {
		qDebug() << "Can't open " << dev;
		return;
	}

	m_notify = new QSocketNotifier(m_fd, QSocketNotifier::Read, this);
	connect(m_notify, SIGNAL(activated(int)), this, SLOT(readEvent()));
	m_ignore = false;

	connect(qApp, SIGNAL(aboutToQuit()), this, SLOT(quit()));
}

LinuxInputEvent::~LinuxInputEvent()
{
	if (m_fd < 0)
		return;

	::close(m_fd);
}

void LinuxInputEvent::quit()
{
	waitForRelease();
}

bool LinuxInputEvent::grab(bool enable)
{
	int ret = ::ioctl(m_fd, EVIOCGRAB, enable);
	return ret == 0;
}

void LinuxInputEvent::ignore(bool ignore)
{
	m_ignore = ignore;
}

void LinuxInputEvent::readEvent()
{
	struct input_event ev_buf[16];
	unsigned char *p_buf = (unsigned char *)ev_buf;
	int size = 0;

	do {
		int ret = read(m_fd, p_buf, sizeof(ev_buf) - size);
		if (ret == 0)
			return;
		if (ret < 0)
			break;

		size += ret;
		p_buf += ret;
	} while (1);

	if (m_ignore)
		return;

	emit triggered();

	for (unsigned int i = 0; i < size / sizeof(struct input_event); i++) {
		recvEvent(&ev_buf[i]);
	}
}

bool LinuxInputEvent::isBuffered()
{
	fd_set fds;
	struct timeval tv = {0, 0};
	if (m_fd < 0)
		return false;

	FD_ZERO(&fds);
	FD_SET(m_fd, &fds);
	if (select(m_fd + 1, &fds, NULL, NULL, &tv) == 1)
		return true;
	return false;
}
