#include "touchevent.h"
#include <QWidget>
#include <QApplication>
#include <QDesktopWidget>
#include <QMouseEvent>
#include <QDebug>
#include <linux/input.h>
#include <fcntl.h>

TouchEvent::TouchEvent(const QString &dev, QObject *parent)
	: LinuxInputEvent(dev, parent)
{
	m_device = dev;
	m_lastbutton = Qt::NoButton;
	m_screen = QApplication::desktop()->size();
}

void TouchEvent::ignore(bool ignore)
{
	m_lastbutton = Qt::NoButton;
	m_widget = 0;
	LinuxInputEvent::ignore(ignore);
}

void TouchEvent::waitForRelease()
{
	while (m_button != Qt::NoButton) {
		qApp->processEvents();
	}
}

void TouchEvent::sendMouseEvent(QEvent::Type type, QPoint globalPos)
{
	if (type == QEvent::MouseButtonPress) {
		m_widget = qApp->widgetAt(globalPos);
	}

	if (!m_widget)
		return;

	QPoint pos = globalPos - m_widget->pos();
	Qt::MouseButton buttons = Qt::NoButton;
//	qDebug() << "geometry:" << m_widget->pos() << m_widget->mapToGlobal(m_widget->pos());

	if (type != QEvent::MouseButtonRelease) {
		buttons = Qt::LeftButton;
	}

//	qDebug() << "event pos:" << pos << " global:" << globalPos << " buttons:" << buttons;
	QMouseEvent event(type, pos, globalPos,
					  Qt::LeftButton, buttons, Qt::NoModifier);
	qApp->sendEvent(m_widget, &event);

	if (type == QEvent::MouseButtonRelease) {
		m_widget = 0;
	}
}

QPoint TouchEvent::trans(QPoint pos)
{
	return QPoint(m_screen.width()-pos.y(), pos.x());
}

void TouchEvent::recvEvent(struct input_event *ev)
{
	switch (ev->type) {
	case EV_SYN: {
//		qDebug("ev:type=EV_SYN");
		QEvent::Type type;

		if (m_lastbutton != m_button) {
			if (m_button != Qt::NoButton) {
				type = QEvent::MouseButtonPress;
//				qDebug() << "Press" << m_pos;
			} else {
				type = QEvent::MouseButtonRelease;
//				qDebug() << "Release" << m_pos;
				m_pos = m_lastPos;
			}
		} else {
			type = QEvent::MouseMove;
//			qDebug() << "Move" << m_pos;
		}

//		QPoint calc_pos(600-m_pos.y(), m_pos.x());
		QPoint calc_pos = trans(m_pos);
//		qDebug() << "calc" << calc_pos;

		sendMouseEvent(type, calc_pos);
		m_lastbutton = m_button;
		m_lastPos = m_pos;

		break;
	}
	case EV_ABS:
//		qDebug("ev:type=EV_ABS");
//		qDebug("ev:code=%x", ev->code);
//		qDebug("ev:value=%x", ev->value);

		switch (ev->code) {
		case ABS_X:
			m_pos.setX(ev->value);
			break;
		case ABS_Y:
			m_pos.setY(ev->value);
			break;
		case ABS_PRESSURE:
			m_button = (ev->value > 10) ? Qt::LeftButton : Qt::NoButton;
			break;
		}
		break;
	default:
//		qDebug("ev:type=%x", ev->type);
		break;
	}
}

